@@ -28,6 +28,7 @@
 
 #include "shell/list-box-helper.h"
 #include "ce-page-ip6.h"
+#include "ui-helpers.h"
 #include <nm-utils.h>
 
 G_DEFINE_TYPE (CEPageIP6, ce_page_ip6, CE_TYPE_PAGE)
@@ -652,12 +653,12 @@ connect_ip6_page (CEPageIP6 *page)
 static gboolean
 ui_to_setting (CEPageIP6 *page)
 {
-        gboolean valid = FALSE;
         const gchar *method;
         gboolean ignore_auto_dns;
         gboolean ignore_auto_routes;
         gboolean never_default;
         GList *children, *l;
+        gboolean ret = TRUE;
 
         if (!gtk_switch_get_active (page->enabled)) {
                 method = NM_SETTING_IP6_CONFIG_METHOD_IGNORE;
@@ -704,26 +705,43 @@ ui_to_setting (CEPageIP6 *page)
 
                 if (!*text_address && !*text_prefix && !*text_gateway) {
                         /* ignore empty rows */
+                        widget_unset_error (GTK_WIDGET (entry));
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "prefix"));
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "gateway"));
                         continue;
                 }
 
                 if (inet_pton (AF_INET6, text_address, &tmp_addr) <= 0) {
-                        goto out;
+                        widget_set_error (GTK_WIDGET (entry));
+                        ret = FALSE;
+                } else {
+                        widget_unset_error (GTK_WIDGET (entry));
                 }
 
                 prefix = strtoul (text_prefix, &end, 10);
                 if (!end || *end || prefix == 0 || prefix > 128) {
-                        goto out;
+                        widget_set_error (g_object_get_data (G_OBJECT (row), "prefix"));
+                        ret = FALSE;
+                } else {
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "prefix"));
                 }
 
                 if (text_gateway && *text_gateway) {
                         if (inet_pton (AF_INET6, text_gateway, &tmp_gateway) <= 0) {
-                                goto out;
+                                widget_set_error (g_object_get_data (G_OBJECT (row), "gateway"));
+                                ret = FALSE;
+                        } else {
+                                if (!IN6_IS_ADDR_UNSPECIFIED (&tmp_gateway))
+                                        have_gateway = TRUE;
+                                widget_unset_error (g_object_get_data (G_OBJECT (row), "gateway"));
                         }
-                        if (!IN6_IS_ADDR_UNSPECIFIED (&tmp_gateway))
-                                have_gateway = TRUE;
+                } else {
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "gateway"));
                 }
 
+                if (!ret)
+                        continue;
+
                 addr = nm_ip6_address_new ();
                 nm_ip6_address_set_address (addr, &tmp_addr);
                 nm_ip6_address_set_prefix (addr, prefix);
@@ -748,14 +766,17 @@ ui_to_setting (CEPageIP6 *page)
                 text = gtk_entry_get_text (entry);
                 if (!*text) {
                         /* ignore empty rows */
+                        widget_unset_error (GTK_WIDGET (entry));
                         continue;
                 }
 
                 if (inet_pton (AF_INET6, text, &tmp_addr) <= 0) {
-                        goto out;
+                        widget_set_error (GTK_WIDGET (entry));
+                        ret = FALSE;
+                } else {
+                        widget_unset_error (GTK_WIDGET (entry));
+                        nm_setting_ip6_config_add_dns (page->setting, &tmp_addr);
                 }
-
-                nm_setting_ip6_config_add_dns (page->setting, &tmp_addr);
         }
         g_list_free (children);
 
@@ -784,20 +805,33 @@ ui_to_setting (CEPageIP6 *page)
 
                 if (!*text_address && !*text_prefix && !*text_gateway && !*text_metric) {
                         /* ignore empty rows */
+                        widget_unset_error (GTK_WIDGET (entry));
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "prefix"));
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "gateway"));
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "metric"));
                         continue;
                 }
 
                 if (inet_pton (AF_INET6, text_address, &dest) <= 0) {
-                        goto out;
+                        widget_set_error (GTK_WIDGET (entry));
+                        ret = FALSE;
+                } else {
+                        widget_unset_error (GTK_WIDGET (entry));
                 }
 
                 prefix = strtoul (text_prefix, &end, 10);
                 if (!end || *end || prefix == 0 || prefix > 128) {
-                        goto out;
+                        widget_set_error (g_object_get_data (G_OBJECT (row), "prefix"));
+                        ret = FALSE;
+                } else {
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "prefix"));
                 }
 
                 if (inet_pton (AF_INET6, text_gateway, &gateway) <= 0) {
-                        goto out;
+                        widget_set_error (g_object_get_data (G_OBJECT (row), "gateway"));
+                        ret = FALSE;
+                } else {
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "gateway"));
                 }
 
                 metric = 0;
@@ -805,10 +839,18 @@ ui_to_setting (CEPageIP6 *page)
                         errno = 0;
                         metric = strtoul (text_metric, NULL, 10);
                         if (errno) {
-                                goto out;
+                                widget_set_error (g_object_get_data (G_OBJECT (row), "metric"));
+                                ret = FALSE;
+                        } else {
+                                widget_unset_error (g_object_get_data (G_OBJECT (row), "metric"));
                         }
+                } else {
+                        widget_unset_error (g_object_get_data (G_OBJECT (row), "metric"));
                 }
 
+                if (!ret)
+                        continue;
+
                 route = nm_ip6_route_new ();
                 nm_ip6_route_set_dest (route, &dest);
                 nm_ip6_route_set_prefix (route, prefix);
@@ -819,6 +861,9 @@ ui_to_setting (CEPageIP6 *page)
         }
         g_list_free (children);
 
+        if (!ret)
+                goto out;
+
         ignore_auto_dns = !gtk_switch_get_active (page->auto_dns);
         ignore_auto_routes = !gtk_switch_get_active (page->auto_routes);
         never_default = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (page->never_default));
@@ -830,11 +875,9 @@ ui_to_setting (CEPageIP6 *page)
                       NM_SETTING_IP6_CONFIG_NEVER_DEFAULT, never_default,
                       NULL);
 
-        valid = TRUE;
-
 out:
 
-        return valid;
+        return ret;
 }
 
 static gboolean
