@@ -26,6 +26,7 @@
 #define GOA_BACKEND_API_IS_SUBJECT_TO_CHANGE
 #include <goabackend/goabackend.h>
 
+#include "shell/list-box-helper.h"
 #include "cc-online-accounts-add-account-dialog.h"
 
 #define BRANDED_PAGE "_branded"
@@ -74,21 +75,10 @@ add_account_dialog_add_account (GoaPanelAddAccountDialog *add_account,
                                 GoaProvider *provider)
 {
   GoaPanelAddAccountDialogPrivate *priv = add_account->priv;
-  GList *children;
-  GList *l;
-  GtkWidget *action_area;
   GtkWidget *vbox;
 
-  action_area = gtk_dialog_get_action_area (GTK_DIALOG (add_account));
   vbox = gtk_dialog_get_content_area (GTK_DIALOG (add_account));
-  children = gtk_container_get_children (GTK_CONTAINER (vbox));
-  for (l = children; l != NULL; l = l->next)
-    {
-      GtkWidget *child = l->data;
-      if (child != action_area)
-        gtk_container_remove (GTK_CONTAINER (vbox), child);
-    }
-  g_list_free (children);
+  gtk_container_foreach (GTK_CONTAINER (vbox), (GtkCallback) gtk_widget_destroy, NULL);
 
   /* This spins gtk_dialog_run() */
   priv->object = goa_provider_add_account (provider,
@@ -113,25 +103,6 @@ list_box_row_activated_cb (GoaPanelAddAccountDialog *add_account, GtkListBoxRow
   add_account_dialog_add_account (add_account, provider);
 }
 
-static void
-update_header_func (GtkListBoxRow  *row,
-                    GtkListBoxRow  *before,
-                    gpointer    user_data)
-{
-  GtkWidget *current;
-
-  if (before == NULL)
-    return;
-
-  current = gtk_list_box_row_get_header (row);
-  if (current == NULL)
-    {
-      current = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
-      gtk_widget_show (current);
-      gtk_list_box_row_set_header (row, current);
-    }
-}
-
 static void
 add_account_dialog_create_group_ui (GoaPanelAddAccountDialog *add_account,
                                     GtkListBox **list_box,
@@ -166,7 +137,7 @@ add_account_dialog_create_group_ui (GoaPanelAddAccountDialog *add_account,
   *list_box = GTK_LIST_BOX (gtk_list_box_new ());
   gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (*list_box));
   gtk_list_box_set_selection_mode (*list_box, GTK_SELECTION_NONE);
-  gtk_list_box_set_header_func (*list_box, update_header_func, NULL, NULL);
+  gtk_list_box_set_header_func (*list_box, cc_list_box_update_header_func, NULL, NULL);
   g_signal_connect_swapped (*list_box, "row-activated",
                             G_CALLBACK (list_box_row_activated_cb), add_account);
 }
@@ -238,7 +209,6 @@ static void
 goa_panel_add_account_dialog_realize (GtkWidget *widget)
 {
   GoaPanelAddAccountDialog *add_account = GOA_PANEL_ADD_ACCOUNT_DIALOG (widget);
-  GtkWidget *button;
   GtkWindow *parent;
 
   parent = gtk_window_get_transient_for (GTK_WINDOW (add_account));
@@ -252,9 +222,6 @@ goa_panel_add_account_dialog_realize (GtkWidget *widget)
     }
 
   GTK_WIDGET_CLASS (goa_panel_add_account_dialog_parent_class)->realize (widget);
-
-  button = gtk_dialog_get_widget_for_response (GTK_DIALOG (add_account), GTK_RESPONSE_CANCEL);
-  gtk_widget_grab_focus (button);
 }
 
 static void
@@ -339,7 +306,7 @@ goa_panel_add_account_dialog_init (GoaPanelAddAccountDialog *add_account)
   priv->branded_list_box = GTK_LIST_BOX (gtk_list_box_new ());
   gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (priv->branded_list_box));
   gtk_list_box_set_selection_mode (priv->branded_list_box, GTK_SELECTION_NONE);
-  gtk_list_box_set_header_func (priv->branded_list_box, update_header_func, NULL, NULL);
+  gtk_list_box_set_header_func (priv->branded_list_box, cc_list_box_update_header_func, NULL, NULL);
   g_signal_connect_swapped (priv->branded_list_box, "row-activated",
                             G_CALLBACK (list_box_row_activated_cb), add_account);
 
@@ -373,9 +340,6 @@ goa_panel_add_account_dialog_init (GoaPanelAddAccountDialog *add_account)
                                       _("Resources"));
 
   gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), BRANDED_PAGE);
-
-  gtk_dialog_add_button (GTK_DIALOG (add_account), _("_Cancel"), GTK_RESPONSE_CANCEL);
-  gtk_dialog_set_default_response (GTK_DIALOG (add_account), GTK_RESPONSE_CANCEL);
 }
 
 static void
@@ -406,7 +370,7 @@ goa_panel_add_account_dialog_class_init (GoaPanelAddAccountDialogClass *klass)
 GtkWidget *
 goa_panel_add_account_dialog_new (GoaClient *client)
 {
-  return g_object_new (GOA_TYPE_PANEL_ADD_ACCOUNT_DIALOG, "client", client, NULL);
+  return g_object_new (GOA_TYPE_PANEL_ADD_ACCOUNT_DIALOG, "client", client, "use-header-bar", TRUE, NULL);
 }
 
 void
